'Paint Demo for DVI and VGA graphics shields
'Copyright (C) 2013 Rayslogic.com, LLC
'MIT License
'Demonstrates mouse cursor, line drawing, bmp saving to SD card


CON 'Crystal settings
  _clkmode = xtal1+PLL16X
  _xinfreq = 5_000_000

OBJ
  lcd: "SSD1963_driver3h"   'NOTE:  Be sure to change pin settings in this driver to match your setup if not using Propeller Platform!
  sd: "fsrw"     'SD card driver
  mouse : "mouse"  

CON 'SD card pin settings      Make sure matches your setup!
  'Propeller Platform Express settings:
  SD_MISO=0
  SD_Clk=1
  SD_MOSI=2
  SD_Cs=3

CON 'Define cursor colors
  mfore=lcd#rgb_LightYellow
  mback=lcd#rgb_black
  
VAR 'buffer for bitmap line of pixels
  long lineBuffer[864*3/4]' buffer for line of an image  'needs to be long aligned
  long BmpPalette[256]

CON 'painting constants
  pwx1=40   'Paint Window x start
  pwdx=450
  pwx2=pwx1+pwdx
  pwy1=100
  pwdy=350
  pwy2=pwy1+pwdy
  cbx1=pwx2+10
  cby1=pwy1
  cbw=30
  cbh=40
  cbdx=35
  cbdy=50

  
VAR  'painting variables
  long fcolor, bcolor, bLineMode, bStillDown  

PUB Main |i
  'start VGA driver
  lcd.start
  lcd.StartMouse(@MouseCursor1,@lineBuffer) 'Mouse support requires mouse draw data and a buffer big enough to hold background

  'start mouse
  mouse.start(12, 13)'(24, 25)
  mouse.bound_limits(0, 0, 0, 639, 479, 0) 'same as with regular mouse
  mouse.bound_scales(1, -1, 0)
  mouse.bound_preset(10,10,0)

  lcd.StartText(@Font1)   'Driver includes an 8x12 font from VGA_HIRES_TEXT (but only characters 0..127)  

  Paint
  

PUB Paint|i,j,k,x,y,LastX,LastY      'Simple paint routine using mouse
  
  fcolor:=lcd#RGB_White
  bcolor:=lcd#RGB_Black
  DrawScreen

  bLineMode:=true
   
  repeat
    if !mouse.button(0)
      quit
  bStillDown:=false
  repeat
    'wait for mouse to be down and moved
    repeat
      x := mouse.bound_x
      y := mouse.bound_y
      if mouse.button(0)
        quit
      elseif mouse.button(1)
        CheckButtons(x,y)
      else
        bStillDown:=false
      'Draw mouse (and save background)
      lcd.ShowMouse(x,y)
      lcd.WaitForRefreshClear(y)
      lcd.HideMouse
      
    if x>pwx2
      'A button may have been pushed
      CheckButtons(x,y)
    elseif bInPaintWindow(x,y)
      'draw point       
      if bLineMode==true
        if bStillDown==true
          lcd.DrawLine(fcolor,x,y,LastX,LastY) 
        else
          lcd.SetPixel(fcolor,x,y)    
      else
        lcd.SetPixel(fcolor,x,y)
      LastX:=x
      LastY:=y
      bStillDown:=true 


PUB CheckButtons(xb,yb)|x,y,i,j,k

  'check for color buttons
  repeat i from 0 to 2
    x:=cbx1+cbdx*i
    if xb>x and xb<(x+cbw)
      repeat j from 0 to 5      
        y:=cby1+cbdy*j
        if yb>y and yb<(y+cbh)
          if mouse.button(0)
            fcolor:=GetColor(i,j)
          if mouse.button(1)
            bcolor:=GetColor(i,j)
            ClearPaint
        
      
      

PRI bInPaintWindow(x,y)  'see if point is in window
  if x<pwx1 or x>pwx2
    return false
  if y<pwy1 or y>pwy2
    return false
  return true
       

PUB DrawScreen|x,y,c,i,j 
  lcd.out(0) 'clear screen  
  lcd.str(string("SSD1963 Paint!",13))
  ClearPaint

  'draw the color buttons
  repeat i from 0 to 2
    repeat j from 0 to 5
      x:=cbx1+cbdx*i
      y:=cby1+cbdy*j
      lcd.DrawSolidRectangle(GetColor(i,j),x,y,x+cbw,y+cbh)
      lcd.DrawRectangle(lcd#rgb_white,x-1,y-1,x+cbw+1,y+cbh+1)
      lcd.DrawRectangle(lcd#rgb_black,x-2,y-2,x+cbw+2,y+cbh+2)   


PUB GetColor(x,y)
  case x
    0:
      case y
        0: return lcd#rgb_white
        1: return lcd#rgb_black
        2: return lcd#rgb_Yellow   
        3: return lcd#rgb_aqua
        4: return lcd#rgb_blue
        5: return lcd#rgb_gray
    1:
      case y
        0: return lcd#rgb_green
        1: return lcd#rgb_lime
        2: return lcd#rgb_maroon   
        3: return lcd#rgb_navy
        4: return lcd#rgb_olive
        5: return lcd#rgb_purple
    2:
      case y
        0: return lcd#rgb_MidnightBlue
        1: return lcd#rgb_MintCream
        2: return lcd#rgb_LightYellow   
        3: return lcd#rgb_red
        4: return lcd#rgb_silver
        5: return lcd#rgb_teal        
           

  

PUB ClearPaint
  lcd.DrawSolidRectangle(bcolor,pwx1,pwy1,pwx2,pwy2) 

Pub PrintScreen(pFilename)|i,j,k,a0,a1,a2,a3,a4,a5  'Grab screen and save as windows bitmap file
  'Mounting SD card now
  repeat   'try a few times to mount the SD card
    i:=\sd.mount_explicit(SD_MISO, SD_CLK, SD_MOSI,SD_CS)
    if i==0
       quit
    waitcnt(cnt+clkfreq/4)
    lcd.out(".")
   
  \sd.popen(pFilename, "w")

  'First, write the header 
  biWidth:=lcd#xPixels
  biHeight:=lcd#yPixels
  bfSize:=biWidth*biHeight*3+54
  bfOffBits:=54
  biBitCount:=24
  biPlanes:=1  
  sd.pwrite(@bfType,2)
  sd.pwrite(@bfSize,4)   
  sd.pwrite(@bfReserved1,2)   
  sd.pwrite(@bfReserved2,2)   
  sd.pwrite(@bfOffBits,4)   
  sd.pwrite(@biSize,4)   
  sd.pwrite(@biWidth,4)   
  sd.pwrite(@biHeight,4)   
  sd.pwrite(@biPlanes,2)   
  sd.pwrite(@biBitCount,2)   
  sd.pwrite(@biCompression,4)   
  sd.pwrite(@biSizeImage,4)   
  sd.pwrite(@biXPelsPerMeter,4)   
  sd.pwrite(@biYPelsPerMeter,4)
  sd.pwrite(@biClrUsed,4)   
  sd.pwrite(@biClrImportant,4)

  'Then, write data
  a0:=0
  a1:=lcd#xPixels
  a3:=1
  a4:=@lineBuffer
  repeat j from  lcd#yPixels-1 to 0
    a2:=j
    'get line of pixels 
    lcd.Setcommand(lcd#ScreenToHub,@a0)  'arg0=x,arg1=w,arg2=y,arg3=h,arg4=pBuffer
    'fix the bits
    lcd.FixColors(@linebuffer,lcd#xPixels)
    'write them to file
    sd.pwrite(@linebuffer,lcd#xPixels *3)

  sd.pclose 

{

PUB Text_Demo|i,j,k 'copies functionality of VGA_Text_Demo.Spin
  'Slightly modified to account for different font being used
  'Note:  This is not VGA Text Demo, is not 2.6kB, and is not using Internal ROM font!
  lcd.StartText   'Driver includes an 8x12 font from VGA_HIRES_TEXT (but only characters 0..127)
  'lcd.SetFont(@Font1)
  lcd.out(0) 'clear screen
  
  lcd.str(string(13,"   VGA Text Demo...",13,13,$C,5," OBJ and VAR require only 2.6KB ",$C,1))
  repeat 14
    lcd.out(" ")
  repeat 4
    repeat i from $0E to 127
      lcd.out(i)
  lcd.str(string($C,6,"     Uses internal ROM font     ",$C,2))
  repeat 3000 'slight mod
    lcd.str(string($A,12,$B,14))
    lcd.hex(i++, 8)

}
 
      
  
    

PUB BMP(bmpX,bmpY,sFilename)|i,j,k,l      'leaving this here so that screen saves can be recalled
'Draw a bitmap file onto the screen
    'Update speed seems to be only limited by SD card reading speed
    'note that bitmaps are stored from bottom line to top line!
    i:=\sd.popen(sFilename,"r") 'open image for reading
    if i<>0
      return i   
    sd.pread(@bfType,2) 'read bmp header
    sd.pread(@bfSize,4) 'read bmp header  
    sd.pread(@bfReserved1,4) 'read bmp header  
    sd.pread(@bfOffBits,16) 'read bmp header
    sd.pread(@biPlanes,4)
    sd.pread(@biCompression,24)

    if (biHeight<1) or (biWidth<1)
      return -1  'some kind of problem

    l:=biHeight+bmpY-1  
      
    case biBitCount
      24:  'true color (3 bytes per pixel)        
        i:=(biWidth*3)//4
        k:=biWidth*3+i     
        repeat j from l to bmpY 
          'process image row by row          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,24,0)
 
      8:  '256-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54
        'read in palette
        sd.pread(@BmpPalette,i)
        i:=biWidth//4
        k:=biWidth+i
        repeat j from l to bmpY          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,8,@BmpPalette) 
      
      4:  '16-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54        
        'read in palette
        sd.pread(@BmpPalette,i)
        'now paint picture
        i:=(biWidth>>1)//4
        k:=(biWidth>>1)+i 
        repeat j from l to bmpY          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,4,@BmpPalette) 

    sd.pclose
    return 0



DAT     'Data for reading bitmap files

BMPHeader  byte 'Mostly using info from here:  http://www.fortunecity.com/skyscraper/windows/364/bmpffrmt.html
bfType byte "B","M"  ' 19778
bfSize long 0
bfReserved1 word 0
bfReserved2 word 0
bfOffBits long 54
biSize long 40
biWidth long 0
biHeight long 10
biPlanes word 1
biBitCount word 24
biCompression long 0
biSizeImage long 0
biXPelsPerMeter long 0
biYPelsPerMeter long 0
biClrUsed long 0
biClrImportant long 0


DAT  'Font:  Custom font located here.  Other fonts can be selected with the SetFont command
'Note:  If you do not need a font, you can save space by commenting out the file command line below
        
font1 long
file "ProgoCo_wHeader.dat"   'nice 8x16 fixed pitch font

DAT 'Define mouse pointer
MouseCursor1 long          'must be long aligned
        long  1,1  'hot spot offset in x,y  
        long  12,19 'width, height of cursor
        long  41   '#quartets to draw
        byte  0,0,1,1  'x,y,#width,increment
        long  mback      'color
        byte  0,1,2,1
        long  mback
        byte  0,2,3,1
        long  mback
        byte  0,3,4,1
        long  mback
        byte  0,4,5,1
        long  mback
        byte  0,5,6,1
        long  mback
        byte  0,6,7,1
        long  mback
        byte  0,7,8,1
        long  mback
        byte  0,8,9,1
        long  mback
        byte  0,9,10,1
        long  mback
        byte  0,10,11,1
        long  mback
        byte  0,11,12,1
        long  mback
        byte  0,12,12,1
        long  mback
        byte  0,13,8,1
        long  mback
        byte  0,14,4,1
        long  mback
        byte  0,15,3,1
        long  mback
        byte  0,16,2,1
        long  mback
        byte  1,2,1,1
        long  mfore
        byte  1,3,2,1
        long  mfore
        byte  1,4,3,1
        long  mfore
        byte  1,5,4,1
        long  mfore
        byte  1,6,5,1
        long  mfore
        byte  1,7,6,1
        long  mfore
        byte  1,8,7,1
        long  mfore
        byte  1,9,8,1
        long  mfore
        byte  1,10,9,1
        long  mfore
        byte  1,11,10,1
        long  mfore
        byte  1,12,6,1
        long  mfore
        byte  1,13,3,1
        long  mfore
        byte  1,14,2,1
        long  mfore
        byte  1,15,1,1
        long  mfore
        byte  5,14,4,1
        long  mback
        byte  5,15,4,1
        long  mback
        byte  6,16,4,1
        long  mback
        byte  6,17,4,1
        long  mback
        byte  7,18,2,1
        long  mback
        byte  5,13,2,1
        long  mfore
        byte  6,14,2,1
        long  mfore
        byte  6,15,2,1
        long  mfore
        byte  7,16,2,1
        long  mfore
        byte  7,17,2,1
        long  mfore
        